function [nodes,values] = solve_pde_per(y,a0,c,mu,hmax)
% SOLVE_PDE_PER method for solving the PDE from Section 4.2
% The method uses the Partial Differential Equation Toolbox of MATLAB.
%
%   Input parameters:
%      y - sampling locations, size [#amount,d]
%      a0,c,mu - PDE parameters
%      hmax - FEM parameter (regulating the grid size)
%
%   Output parameters:
%      nodes - nodes of the FEM
%      values - function values of the solution, size [#amount,#nodes]
%
% Copyright (c) 2021 Fabian Taubert

[n,d] = size(y);
model = createpde();
R = [3,4,0,1,1,0,0,0,1,1]';
gm = R;
sf = 'R';
ns = char('R');
g = decsg(gm,sf,ns);
geometryFromEdges(model,g);
applyBoundaryCondition(model,'dirichlet','Edge',1:model.Geometry.NumEdges,'u',0);
specifyCoefficients(model,'m',0,'d',0,'c',@(location, state) dif_coef_pde_per(location,state,y(1,:),d,a0,c,mu),'a',0,'f',@(location,state) location.y);
generateMesh(model,'Hmax',hmax);
results = solvepde(model);
nodes = results.Mesh.Nodes;
grid_size = size(nodes,2);
values = zeros(n,grid_size);
values(1,:) = results.NodalSolution;

parfor j = 2:n
    model = createpde();
    geometryFromEdges(model,g);
    applyBoundaryCondition(model,'dirichlet','Edge',1:model.Geometry.NumEdges,'u',0);
    specifyCoefficients(model,'m',0,'d',0,'c',@(location, state) dif_coef_pde_per(location,state,y(j,:),d,a0,c,mu),'a',0,'f',@(location,state) location.y);
    generateMesh(model,'Hmax',hmax);
    results = solvepde(model);
    values(j,:) = results.NodalSolution;
end
end

function vlt=dif_coef_pde_per(location,state,y,d,a0,c,mu)
vlt = a0;
 for j=1:d
  vlt=vlt+1/sqrt(6).*sin(2*pi*y(j)).*(c/(j^mu)).*sin(j*pi*location.x).*sin(j*pi*location.y);
 end
end